@@ -1,6 +1,6 @@
 from . import register_class
 from ..container import Container, Data, MultiContainerInterface
-from ..utils import docval, call_docval_func, popargs
+from ..utils import docval, popargs, AllowPositional
 
 
 @register_class('SimpleMultiContainer')
@@ -15,8 +15,9 @@ class SimpleMultiContainer(MultiContainerInterface):
 
     @docval({'name': 'name', 'type': str, 'doc': 'the name of this container'},
             {'name': 'containers', 'type': (list, tuple), 'default': None,
-             'doc': 'the Container or Data objects in this file'})
+             'doc': 'the Container or Data objects in this file'},
+            allow_positional=AllowPositional.WARNING)
     def __init__(self, **kwargs):
         containers = popargs('containers', kwargs)
-        call_docval_func(super().__init__, kwargs)
+        super().__init__(**kwargs)
         self.containers = containers
