@@ -12,5 +12,5 @@ class SimpleMultiContainerRoundTrip(H5RoundTripMixin, TestCase):
             Data('data1', [0, 1, 2, 3, 4]),
             Data('data2', [0.0, 1.0, 2.0, 3.0, 4.0]),
         ]
-        multi_container = SimpleMultiContainer('multi', containers)
+        multi_container = SimpleMultiContainer(name='multi', containers=containers)
         return multi_container
